@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology.
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,114 +18,87 @@
 #include "meep_internals.hpp"
 #include "bicgstab.hpp"
 
+using namespace std;
+
 namespace meep {
 
-static void fields_to_array(const fields &f, complex<double> *x)
+static void fields_to_array(const fields &f, complex<realnum> *x)
 {
-  int ix = 0;
+  size_t ix = 0;
   for (int i=0;i<f.num_chunks;i++)
     if (f.chunks[i]->is_mine())
       FOR_COMPONENTS(c)
         if (is_D(c) || is_B(c)) {
-	  double *fr, *fi;
-	  if ((fr = f.chunks[i]->f[c][0]) &&
-	      (fi = f.chunks[i]->f[c][1]))
-	    LOOP_OVER_VOL_OWNED(f.chunks[i]->v, c, idx)
+	  realnum *fr, *fi;
+#define COPY_FROM_FIELD(fld)					\
+	  if ((fr = f.chunks[i]->fld[0]) &&			\
+	      (fi = f.chunks[i]->fld[1]))			\
+	    LOOP_OVER_VOL_OWNED(f.chunks[i]->gv, c, idx)		\
 	      x[ix++] = complex<double>(fr[idx], fi[idx]);
+	  COPY_FROM_FIELD(f[c]);
+	  COPY_FROM_FIELD(f_u[c]);
+	  COPY_FROM_FIELD(f_cond[c]);
+	  component c2 = field_type_component(is_D(c) ? E_stuff : H_stuff, c);
+	  COPY_FROM_FIELD(f_w[c2]);
+	  if (f.chunks[i]->f_w[c2][0]) COPY_FROM_FIELD(f[c2]);
+#undef COPY_FROM_FIELD
 	}
-
-  for (int i=0;i<f.num_chunks;i++)
-    if (f.chunks[i]->is_mine()) {
-      bool have_pml = false;
-      LOOP_OVER_FIELD_DIRECTIONS(f.chunks[i]->v.dim, d)
-	if (f.chunks[i]->s->sigsize[d] > 1)
-	  have_pml = true;
-      if (have_pml) FOR_COMPONENTS(c)
-        if (is_electric(c) || is_magnetic(c)) {
-	  double *fr, *fi;
-	  if ((fr = f.chunks[i]->f[c][0]) &&
-	      (fi = f.chunks[i]->f[c][1]))
-	    LOOP_OVER_VOL_OWNED(f.chunks[i]->v, c, idx)
-	      x[ix++] = complex<double>(fr[idx], fi[idx]);
-	}
-    }
 }
-  
-static void array_to_fields(const complex<double> *x, fields &f)
+
+static void array_to_fields(const complex<realnum> *x, fields &f)
 {
-  int ix = 0;
+  size_t ix = 0;
   for (int i=0;i<f.num_chunks;i++)
     if (f.chunks[i]->is_mine())
       FOR_COMPONENTS(c)
         if (is_D(c) || is_B(c)) {
-	  double *fr, *fi;
-	  if ((fr = f.chunks[i]->f[c][0]) &&
-	      (fi = f.chunks[i]->f[c][1]))
-	    LOOP_OVER_VOL_OWNED(f.chunks[i]->v, c, idx) {
-	      fr[idx] = real(x[ix]);
-	      fi[idx] = imag(x[ix++]);
+	  realnum *fr, *fi;
+#define COPY_TO_FIELD(fld)					\
+	  if ((fr = f.chunks[i]->fld[0]) &&			\
+	      (fi = f.chunks[i]->fld[1]))			\
+	    LOOP_OVER_VOL_OWNED(f.chunks[i]->gv, c, idx) {	\
+	      fr[idx] = real(x[ix]);				\
+	      fi[idx] = imag(x[ix++]);				\
 	    }
+	  COPY_TO_FIELD(f[c]);
+	  COPY_TO_FIELD(f_u[c]);
+	  COPY_TO_FIELD(f_cond[c]);
+	  component c2 = field_type_component(is_D(c) ? E_stuff : H_stuff, c);
+	  COPY_TO_FIELD(f_w[c2]);
+	  if (f.chunks[i]->f_w[c2][0]) COPY_TO_FIELD(f[c2]);
+#undef COPY_TO_FIELD
 	}
 
   f.step_boundaries(D_stuff);
-  f.step_boundaries(B_stuff);
-
-  for (int i=0;i<f.num_chunks;i++)
-    if (f.chunks[i]->is_mine()) {
-      bool have_pml = false;
-      LOOP_OVER_FIELD_DIRECTIONS(f.chunks[i]->v.dim, d)
-	if (f.chunks[i]->s->sigsize[d] > 1)
-	  have_pml = true;
-      FOR_COMPONENTS(c) {
-        if (is_electric(c) || is_magnetic(c)) {
-	  if (have_pml) { // in PML regions, E/H fields are unknowns
-	    double *fr, *fi;
-	    if ((fr = f.chunks[i]->f[c][0]) &&
-		(fi = f.chunks[i]->f[c][1]))
-	      LOOP_OVER_VOL_OWNED(f.chunks[i]->v, c, idx) {
-	        fr[idx] = real(x[ix]);
-	        fi[idx] = imag(x[ix++]);
-	      }
-	  }
-	  else if (is_electric(c)) {
-	    src_vol *save_src = f.chunks[i]->d_sources;
-	    f.chunks[i]->d_sources = 0; // disable sources
-	    f.chunks[i]->update_e_from_d();
-	    f.chunks[i]->d_sources = save_src;
-	  }
-	  else if (is_magnetic(c)) {
-	    src_vol *save_src = f.chunks[i]->b_sources;
-	    f.chunks[i]->b_sources = 0; // disable sources
-	    f.chunks[i]->update_h_from_b();
-	    f.chunks[i]->b_sources = save_src;
-	  }
-	}
-      }
-    }
-
+  f.update_eh(E_stuff, true);
   f.step_boundaries(E_stuff);
-  f.step_boundaries(H_stuff);
+
+  /* done in f.step before updating D:
+  f.step_boundaries(B_stuff);
+  f.update_eh(H_stuff);
+  f.step_boundaries(H_stuff); */
 }
 
 typedef struct {
-  int n;
+  size_t n;
   fields *f;
   complex<double> iomega;
   int iters;
 } fieldop_data;
 
-static void fieldop(const double *xr, double *yr, void *data_)
+static void fieldop(const realnum *xr, realnum *yr, void *data_)
 {
-  const complex<double> *x = reinterpret_cast<const complex<double>*>(xr);
-  complex<double> *y = reinterpret_cast<complex<double>*>(yr);
+  const complex<realnum> *x = reinterpret_cast<const complex<realnum>*>(xr);
+  complex<realnum> *y = reinterpret_cast<complex<realnum>*>(yr);
   fieldop_data *data = (fieldop_data *) data_;
   array_to_fields(x, *data->f);
   data->f->step();
   fields_to_array(*data->f, y);
-  int n = data->n;
-  double dt_inv = 1.0 / data->f->dt;
-  complex<double> iomega = data->iomega;
-  for (int i = 0; i < n; ++i) y[i] = (y[i] - x[i]) * dt_inv + iomega * x[i];
+  size_t n = data->n;
+  realnum dt_inv = 1.0 / data->f->dt;
+  complex<realnum> iomega = complex<realnum>(real(data->iomega),
+					     imag(data->iomega));
+  for (size_t i = 0; i < n; ++i) y[i] = (y[i] - x[i]) * dt_inv + iomega * x[i];
   data->iters++;
 }
 
@@ -143,46 +116,54 @@ bool fields::solve_cw(double tol, int maxiters, complex<double> frequency,
 		      int L) {
   if (is_real) abort("solve_cw is incompatible with use_real_fields()");
   if (L < 1) abort("solve_cw called with L = %d < 1", L);
+  int tsave = t; // save time (gets incremented by iterations)
+
+  set_solve_cw_omega(2*pi*frequency);
 
   step(); // step once to make sure everything is allocated
 
-  int N = 0; // size of linear system (on this processor, at least)
+  size_t N = 0; // size of linear system (on this processor, at least)
   for (int i=0;i<num_chunks;i++)
     if (chunks[i]->is_mine()) {
-      bool have_pml = false;
-      LOOP_OVER_FIELD_DIRECTIONS(chunks[i]->v.dim, d)
-	if (chunks[i]->s->sigsize[d] > 1)
-	  have_pml = true;
       FOR_COMPONENTS(c)
-	if (chunks[i]->f[c][0] && (is_D(c) || is_B(c)))
-	  N += 2 * chunks[i]->v.nowned(c) * (1 + have_pml);
+	if (chunks[i]->f[c][0] && (is_D(c) || is_B(c))) {
+	  component c2 = field_type_component(is_D(c) ? E_stuff : H_stuff, c);
+	  /* unknowns are just D and B in non-PML regions, but in PML
+	     regions the E, U, W, and C fields are also unknowns (in
+	     principle, we might be able to compute these extra fields
+	     in frequency domain via scalinb by the appropriate s
+	     factors, rather than storing them, but I had some
+	     problems getting that working) */
+	  N += 2 * chunks[i]->gv.nowned(c) *
+	    (1 + (chunks[i]->f_u[c][0] != NULL)
+	     + (chunks[i]->f_w[c2][0] != NULL) * 2
+	     + (chunks[i]->f_cond[c][0] != NULL));
+	}
     }
 
-  int nwork = bicgstabL(L, N, 0, 0, 0, 0, tol, &maxiters, 0, true);
-  double *work = new double[nwork + 2*N];
-  complex<double> *x = reinterpret_cast<complex<double>*>(work + nwork);
-  complex<double> *b = reinterpret_cast<complex<double>*>(work + nwork + N);
-
-  int tsave = t; // save time (gets incremented by iterations)
+  size_t nwork = (size_t) bicgstabL(L, N, 0, 0, 0, 0, tol, &maxiters, 0, true);
+  realnum *work = new realnum[nwork + 2*N];
+  complex<realnum> *x = reinterpret_cast<complex<realnum>*>(work + nwork);
+  complex<realnum> *b = reinterpret_cast<complex<realnum>*>(work + nwork + N);
 
   fields_to_array(*this, x); // initial guess = initial fields
 
   // get J amplitudes from current time step
   zero_fields(); // note that we've saved the fields in x above
   calc_sources(time());
-  step_b_source();
+  step_source(B_stuff, true);
   step_boundaries(B_stuff);
-  update_h_from_b();
+  update_eh(H_stuff);
   calc_sources(time() + 0.5*dt);
-  step_d_source(1);
+  step_source(D_stuff, true);
   step_boundaries(D_stuff);
-  update_e_from_d();
+  update_eh(E_stuff);
   fields_to_array(*this, b);
   double mdt_inv = -1.0 / dt;
-  for (int i = 0; i < N/2; ++i) b[i] *= mdt_inv;
+  for (size_t i = 0; i < N/2; ++i) b[i] *= mdt_inv;
   {
     double bmax = 0;
-    for (int i = 0; i < N/2; ++i) {
+    for (size_t i = 0; i < N/2; ++i) {
       double babs = abs(b[i]);
       if (babs > bmax) bmax = babs;
     }
@@ -196,25 +177,25 @@ bool fields::solve_cw(double tol, int maxiters, complex<double> frequency,
 		 * (1.0 / dt));
   data.iters = 0;
 
-  bool save_disable_sources = disable_sources;
-  disable_sources = true;
-
-  int ierr = bicgstabL(L, N, reinterpret_cast<double*>(x),
-		       fieldop, &data, reinterpret_cast<double*>(b),
+  int ierr = (int) bicgstabL(L, N, reinterpret_cast<realnum*>(x),
+		       fieldop, &data, reinterpret_cast<realnum*>(b),
 		       tol, &maxiters, work, quiet);
 
   if (!quiet) {
-    master_printf("Finished solve_cw after %d steps and %d CG iters.\n", 
+    master_printf("Finished solve_cw after %d steps and %d CG iters.\n",
 		  data.iters, maxiters);
     if (ierr)
       master_printf(" -- CONVERGENCE FAILURE (%d) in solve_cw!\n", ierr);
   }
 
   array_to_fields(x, *this);
-  
+  step(); // ensure H/B are updated and synced with E/D
+
   delete[] work;
   t = tsave;
-  disable_sources = save_disable_sources;
+
+  unset_solve_cw_omega();
+  update_dfts();
 
   return !ierr;
 }
