@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -18,12 +18,15 @@
 /* HDF5 output of fields and arbitrary functions thereof.  Works
    very similarly to integrate.cpp (using fields::loop_in_chunks). */
 
+#include <algorithm>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
 
 #include "meep_internals.hpp"
 
+using namespace std;
+
 namespace meep {
 
 /***************************************************************************/
@@ -33,8 +36,8 @@ typedef struct {
   h5file *file;
   ivec min_corner, max_corner;
   int num_chunks;
-  double *buf;
-  int bufsz;
+  realnum *buf;
+  size_t bufsz;
   int rank;
   direction ds[3];
 
@@ -46,14 +49,13 @@ typedef struct {
   component *cS;
   complex<double> *ph;
   complex<double> *fields;
-  int *offsets;
+  ptrdiff_t *offsets;
   int ninveps;
   component inveps_cs[3];
   direction inveps_ds[3];
   int ninvmu;
   component invmu_cs[3];
   direction invmu_ds[3];
-  complex<long double> sum;
   field_function fun;
   void *fun_data_;
 } h5_output_data;
@@ -76,8 +78,8 @@ static void h5_findsize_chunkloop(fields_chunk *fc, int ichnk, component cgrid,
   data->min_corner = min(data->min_corner, min(isS, ieS));
   data->max_corner = max(data->max_corner, max(isS, ieS));
   data->num_chunks++;
-  int bufsz = 1;
-  LOOP_OVER_DIRECTIONS(fc->v.dim, d)
+  size_t bufsz = 1;
+  LOOP_OVER_DIRECTIONS(fc->gv.dim, d)
     bufsz *= (ie.in_direction(d) - is.in_direction(d)) / 2 + 1;
   data->bufsz = max(data->bufsz, bufsz);
 }
@@ -97,21 +99,21 @@ static void h5_output_chunkloop(fields_chunk *fc, int ichnk, component cgrid,
   //-----------------------------------------------------------------------//
   // Find output chunk dimensions and strides, etc.
 
-  int start[3]={0,0,0}, count[3]={1,1,1};
-  int offset[3]={0,0,0}, stride[3]={1,1,1};
+  size_t start[3]={0,0,0}, count[3]={1,1,1};
+  ptrdiff_t offset[3]={0,0,0}, stride[3]={1,1,1};
 
   ivec isS = S.transform(is, sn) + shift;
   ivec ieS = S.transform(ie, sn) + shift;
-  
+
   // figure out what yucky_directions (in LOOP_OVER_IVECS)
   // correspond to what directions in the transformed vectors (in output).
-  ivec permute(zero_ivec(fc->v.dim));
-  for (int i = 0; i < 3; ++i) 
-    permute.set_direction(fc->v.yucky_direction(i), i);
+  ivec permute(zero_ivec(fc->gv.dim));
+  for (int i = 0; i < 3; ++i)
+    permute.set_direction(fc->gv.yucky_direction(i), i);
   permute = S.transform_unshifted(permute, sn);
   LOOP_OVER_DIRECTIONS(permute.dim, d)
     permute.set_direction(d, abs(permute.in_direction(d)));
-  
+
   // compute the size of the chunk to output, and its strides etc.
   for (int i = 0; i < data->rank; ++i) {
     direction d = data->ds[i];
@@ -127,55 +129,55 @@ static void h5_output_chunkloop(fields_chunk *fc, int ichnk, component cgrid,
     offset[j] *= stride[j];
     if (offset[j]) stride[j] *= -1;
   }
-  
+
   //-----------------------------------------------------------------------//
   // Compute the function to output, exactly as in fields::integrate,
   // except that here we store its values in a buffer instead of integrating.
 
-  int *off = data->offsets;
+  ptrdiff_t *off = data->offsets;
   component *cS = data->cS;
   complex<double> *fields = data->fields, *ph = data->ph;
-  complex<long double> sum = 0.0;
   const component *iecs = data->inveps_cs;
   const direction *ieds = data->inveps_ds;
-  int ieos[6];
+  ptrdiff_t ieos[6];
   const component *imcs = data->invmu_cs;
   const direction *imds = data->invmu_ds;
-  int imos[6];
+  ptrdiff_t imos[6];
 
   for (int i = 0; i < data->num_fields; ++i) {
     cS[i] = S.transform(data->components[i], -sn);
     if (cS[i] == Dielectric || cS[i] == Permeability)
       ph[i] = 1.0;
     else {
-      fc->v.yee2diel_offsets(cS[i], off[2*i], off[2*i+1]);
+      fc->gv.yee2cent_offsets(cS[i], off[2*i], off[2*i+1]);
       ph[i] = shift_phase * S.phase_shift(cS[i], sn);
     }
   }
   for (int k = 0; k < data->ninveps; ++k)
-    fc->v.yee2diel_offsets(iecs[k], ieos[2*k], ieos[2*k+1]);
+    fc->gv.yee2cent_offsets(iecs[k], ieos[2*k], ieos[2*k+1]);
   for (int k = 0; k < data->ninvmu; ++k)
-    fc->v.yee2diel_offsets(imcs[k], imos[2*k], imos[2*k+1]);
+    fc->gv.yee2cent_offsets(imcs[k], imos[2*k], imos[2*k+1]);
 
-  vec rshift(shift * (0.5*fc->v.inva));
-  LOOP_OVER_IVECS(fc->v, is, ie, idx) {
-    IVEC_LOOP_LOC(fc->v, loc);
+  vec rshift(shift * (0.5*fc->gv.inva));
+  LOOP_OVER_IVECS(fc->gv, is, ie, idx) {
+    IVEC_LOOP_LOC(fc->gv, loc);
     loc = S.transform(loc, sn) + rshift;
 
     for (int i = 0; i < data->num_fields; ++i) {
       if (cS[i] == Dielectric) {
 	double tr = 0.0;
-	for (int k = 0; k < data->ninveps; ++k)
-	  tr += (fc->s->inveps[iecs[k]][ieds[k]][idx]
-		 + fc->s->inveps[iecs[k]][ieds[k]][idx+ieos[2*k]]
-		 + fc->s->inveps[iecs[k]][ieds[k]][idx+ieos[1+2*k]]
-		 + fc->s->inveps[iecs[k]][ieds[k]][idx+ieos[2*k]+ieos[1+2*k]]);
+	for (int k = 0; k < data->ninveps; ++k) {
+	  const realnum *ie = fc->s->chi1inv[iecs[k]][ieds[k]];
+	  if (ie) tr += (ie[idx] + ie[idx+ieos[2*k]] + ie[idx+ieos[1+2*k]]
+			 + ie[idx+ieos[2*k]+ieos[1+2*k]]);
+	  else tr += 4; // default inveps == 1
+	}
 	fields[i] = (4 * data->ninveps) / tr;
       }
       else if (cS[i] == Permeability) {
 	double tr = 0.0;
 	for (int k = 0; k < data->ninvmu; ++k) {
-	  const double *im = fc->s->invmu[imcs[k]][imds[k]];
+	  const realnum *im = fc->s->chi1inv[imcs[k]][imds[k]];
 	  if (im) tr += (im[idx] + im[idx+imos[2*k]] + im[idx+imos[1+2*k]]
 			 + im[idx+imos[2*k]+imos[1+2*k]]);
 	  else tr += 4; // default invmu == 1
@@ -197,8 +199,8 @@ static void h5_output_chunkloop(fields_chunk *fc, int ichnk, component cgrid,
     }
 
     complex<double> fun = data->fun(fields, loc, data->fun_data_);
-    int idx2 = ((((offset[0] + offset[1] + offset[2])
-		  + loop_i1 * stride[0]) 
+    ptrdiff_t idx2 = ((((offset[0] + offset[1] + offset[2])
+		  + loop_i1 * stride[0])
 		 + loop_i2 * stride[1]) + loop_i3 * stride[2]);
     data->buf[idx2] = data->reim ? imag(fun) : real(fun);
   }
@@ -211,21 +213,21 @@ static void h5_output_chunkloop(fields_chunk *fc, int ichnk, component cgrid,
 void fields::output_hdf5(h5file *file, const char *dataname,
 			 int num_fields, const component *components,
 			 field_function fun, void *fun_data_, int reim,
-			 const geometric_volume &where,
+			 const volume &where,
 			 bool append_data,
                          bool single_precision) {
   am_now_working_on(FieldOutput);
   h5_output_data data;
 
   data.file = file;
-  data.min_corner = v.round_vec(where.get_max_corner()) + one_ivec(v.dim);
-  data.max_corner = v.round_vec(where.get_min_corner()) - one_ivec(v.dim);
+  data.min_corner = gv.round_vec(where.get_max_corner()) + one_ivec(gv.dim);
+  data.max_corner = gv.round_vec(where.get_min_corner()) - one_ivec(gv.dim);
   data.num_chunks = 0;
   data.bufsz = 0;
   data.reim = reim;
 
-  loop_in_chunks(h5_findsize_chunkloop, (void *) &data, 
-	    where, Dielectric, true, true);
+  loop_in_chunks(h5_findsize_chunkloop, (void *) &data,
+	    where, Centered, true, true);
 
   file->prevent_deadlock(); // can't hold a lock since *_to_all is collective
   data.max_corner = max_to_all(data.max_corner);
@@ -234,11 +236,12 @@ void fields::output_hdf5(h5file *file, const char *dataname,
   if (data.num_chunks == 0 || !(data.min_corner <= data.max_corner))
     return; // no data to write;
 
-  int rank = 0, dims[3];
-  LOOP_OVER_DIRECTIONS(v.dim, d) {
+  int rank = 0;
+  size_t dims[3];
+  LOOP_OVER_DIRECTIONS(gv.dim, d) {
     if (rank >= 3) abort("too many dimensions in output_hdf5");
-    int n = (data.max_corner.in_direction(d)
-	     - data.min_corner.in_direction(d)) / 2 + 1;
+    size_t n = std::max(0, (data.max_corner.in_direction(d) - data.min_corner.in_direction(d)) / 2 + 1);
+
     if (n > 1) {
       data.ds[rank] = d;
       dims[rank++] = n;
@@ -249,14 +252,13 @@ void fields::output_hdf5(h5file *file, const char *dataname,
   file->create_or_extend_data(dataname, rank, dims,
                               append_data, single_precision);
 
-  data.buf = new double[data.bufsz];
+  data.buf = new realnum[data.bufsz];
 
   data.num_fields = num_fields;
   data.components = components;
   data.cS = new component[num_fields];
   data.ph = new complex<double>[num_fields];
   data.fields = new complex<double>[num_fields];
-  data.sum = 0;
   data.fun = fun;
   data.fun_data_ = fun_data_;
 
@@ -265,33 +267,33 @@ void fields::output_hdf5(h5file *file, const char *dataname,
   bool needs_dielectric = false;
   for (int i = 0; i < num_fields; ++i)
     if (components[i] == Dielectric) { needs_dielectric = true; break; }
-  if (needs_dielectric) 
-    FOR_ELECTRIC_COMPONENTS(c) if (v.has_field(c)) {
+  if (needs_dielectric)
+    FOR_ELECTRIC_COMPONENTS(c) if (gv.has_field(c)) {
       if (data.ninveps == 3) abort("more than 3 field components??");
       data.inveps_cs[data.ninveps] = c;
       data.inveps_ds[data.ninveps] = component_direction(c);
       ++data.ninveps;
     }
-  
+
   /* compute inverse-mu directions for computing Permeability fields */
   data.ninvmu = 0;
   bool needs_permeability = false;
   for (int i = 0; i < num_fields; ++i)
     if (components[i] == Permeability) { needs_permeability = true; break; }
-  if (needs_permeability) 
-    FOR_MAGNETIC_COMPONENTS(c) if (v.has_field(c)) {
+  if (needs_permeability)
+    FOR_MAGNETIC_COMPONENTS(c) if (gv.has_field(c)) {
       if (data.ninvmu == 3) abort("more than 3 field components??");
       data.invmu_cs[data.ninvmu] = c;
       data.invmu_ds[data.ninvmu] = component_direction(c);
       ++data.ninvmu;
     }
-  
-  data.offsets = new int[2 * num_fields];
+
+  data.offsets = new ptrdiff_t[2 * num_fields];
   for (int i = 0; i < 2 * num_fields; ++i)
     data.offsets[i] = 0;
-  
-  loop_in_chunks(h5_output_chunkloop, (void *) &data, 
-		 where, Dielectric, true, true);
+
+  loop_in_chunks(h5_output_chunkloop, (void *) &data,
+		 where, Centered, true, true);
 
   delete[] data.offsets;
   delete[] data.fields;
@@ -307,7 +309,7 @@ void fields::output_hdf5(h5file *file, const char *dataname,
 void fields::output_hdf5(const char *dataname,
                          int num_fields, const component *components,
                          field_function fun, void *fun_data_,
-                         const geometric_volume &where,
+                         const volume &where,
 			 h5file *file,
                          bool append_data,
                          bool single_precision,
@@ -319,17 +321,17 @@ void fields::output_hdf5(const char *dataname,
     file = open_h5file(dataname, h5file::WRITE, prefix, true);
 
   if (real_part_only) {
-    output_hdf5(file, dataname, num_fields, components, fun, fun_data_, 
+    output_hdf5(file, dataname, num_fields, components, fun, fun_data_,
 		0, where, append_data, single_precision);
   }
   else {
     int len = strlen(dataname) + 5;
     char *dataname2 = new char[len];
     snprintf(dataname2, len, "%s%s", dataname, ".r");
-    output_hdf5(file, dataname2, num_fields, components, fun, fun_data_, 
+    output_hdf5(file, dataname2, num_fields, components, fun, fun_data_,
 		0, where, append_data, single_precision);
     snprintf(dataname2, len, "%s%s", dataname, ".i");
-    output_hdf5(file, dataname2, num_fields, components, fun, fun_data_, 
+    output_hdf5(file, dataname2, num_fields, components, fun, fun_data_,
 		1, where, append_data, single_precision);
     delete[] dataname2;
   }
@@ -354,7 +356,7 @@ static complex<double> rintegrand_fun(const complex<double> *fields,
 void fields::output_hdf5(const char *dataname,
                          int num_fields, const component *components,
                          field_rfunction fun, void *fun_data_,
-                         const geometric_volume &where,
+                         const volume &where,
 			 h5file *file,
                          bool append_data,
                          bool single_precision,
@@ -383,18 +385,18 @@ static complex<double> component_fun(const complex<double> *fields,
 }
 
 void fields::output_hdf5(component c,
-			 const geometric_volume &where,
+			 const volume &where,
 			 h5file *file,
 			 bool append_data,
                          bool single_precision,
 			 const char *prefix) {
   if (is_derived(int(c))) {
-    output_hdf5(derived_component(c), 
+    output_hdf5(derived_component(c),
 		where, file, append_data, single_precision, prefix);
     return;
   }
 
-  if (coordinate_mismatch(v.dim, c)) return;
+  if (coordinate_mismatch(gv.dim, c)) return;
 
   char dataname[256];
   bool has_imag = !is_real && c != Dielectric && c != Permeability;
@@ -418,22 +420,22 @@ void fields::output_hdf5(component c,
 /***************************************************************************/
 
 void fields::output_hdf5(derived_component c,
-			 const geometric_volume &where,
+			 const volume &where,
 			 h5file *file,
 			 bool append_data,
                          bool single_precision,
 			 const char *prefix) {
   if (!is_derived(int(c))) {
-    output_hdf5(component(c), 
+    output_hdf5(component(c),
 		where, file, append_data, single_precision, prefix);
     return;
   }
 
-  if (coordinate_mismatch(v.dim, c)) return;
+  if (coordinate_mismatch(gv.dim, c)) return;
 
   int nfields;
   component cs[12];
-  field_rfunction fun = derived_component_func(c, v, nfields, cs);
+  field_rfunction fun = derived_component_func(c, gv, nfields, cs);
 
   output_hdf5(component_name(c), nfields, cs, fun, &nfields, where,
 	      file, append_data, single_precision, prefix);
@@ -441,7 +443,7 @@ void fields::output_hdf5(derived_component c,
 
 /***************************************************************************/
 
-const char *fields::h5file_name(const char *name, 
+const char *fields::h5file_name(const char *name,
 				const char *prefix, bool timestamp)
 {
   const int buflen = 1024;
@@ -453,7 +455,7 @@ const char *fields::h5file_name(const char *name,
       snprintf(time_step_string, 32, "-%09.2f", time());
     else
       snprintf(time_step_string, 32, "-%09d", t);
-  } 
+  }
 
   snprintf(filename, buflen, "%s/" "%s%s" "%s" "%s" ".h5",
 	   outdir,
@@ -462,7 +464,7 @@ const char *fields::h5file_name(const char *name,
   return filename;
 }
 
-h5file *fields::open_h5file(const char *name, h5file::access_mode mode, 
+h5file *fields::open_h5file(const char *name, h5file::access_mode mode,
 			    const char *prefix, bool timestamp)
 {
   const char *filename = h5file_name(name, prefix, timestamp);
