@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -30,17 +30,22 @@
 #  include <gsl/gsl_sf_bessel.h>
 #endif
 
+using namespace std;
+
 namespace meep {
 
-static double J(int m, double kr) {
-#ifdef HAVE_LIBGSL
+#define J BesselJ
+double J(int m, double kr) {
+#if defined(HAVE_LIBGSL)
   return gsl_sf_bessel_Jn(m, kr);
+#elif defined(HAVE_JN)
+  return jn(m, kr); // POSIX/BSD jn function
 #else
   abort("not compiled with GSL, required for Bessel functions");
   return 0;
 #endif
 }
-static double Jprime(int m, double kr) { 
+static double Jprime(int m, double kr) {
   if (m) return 0.5*(J(m-1,kr)-J(m+1,kr));
   else return -J(1,kr);
 }
@@ -48,6 +53,7 @@ static double Jroot(int m, int n) {
 #ifdef HAVE_LIBGSL
   return gsl_sf_bessel_zero_Jnu(m, n+1);
 #else
+  (void) m; (void) n;
   abort("not compiled with GSL, required for Bessel functions");
   return 0;
 #endif
@@ -68,11 +74,11 @@ static double Jmax(int m, int n) {
 
 static double ktrans, kax;
 static int m_for_J;
-static complex<double> JJ(const vec &v) {
-  return polar(J(m_for_J, ktrans*v.r()),kax*v.r());
+static complex<double> JJ(const vec &pt) {
+  return polar(J(m_for_J, ktrans*pt.r()),kax*pt.r());
 }
-static complex<double> JP(const vec &v) {
-  return polar(Jprime(m_for_J, ktrans*v.r()),kax*v.r());
+static complex<double> JP(const vec &pt) {
+  return polar(Jprime(m_for_J, ktrans*pt.r()),kax*pt.r());
 }
 
 void fields::initialize_with_nth_te(int np0) {
@@ -85,7 +91,7 @@ void fields_chunk::initialize_with_nth_te(int np0, double kz) {
   const int im = int(m);
   const int n = (im==0) ? np0 - 0 : np0 - 1;
   const double rmax = Jmax(im,n);
-  ktrans = rmax*a/v.nr();
+  ktrans = rmax*a/gv.nr();
   kax = kz*2*pi/a;
   m_for_J = im;
   initialize_field(Hz, JJ);
@@ -102,7 +108,7 @@ void fields_chunk::initialize_with_nth_tm(int np1, double kz) {
   const int im = int(m);
   const int n = np1 - 1;
   const double rroot = Jroot(im,n);
-  ktrans = rroot*a/v.nr();
+  ktrans = rroot*a/gv.nr();
   kax = kz*2*pi/a;
   m_for_J = im;
   initialize_field(Ez, JJ);
@@ -121,16 +127,21 @@ void fields::initialize_field(component c, complex<double> func(const vec &)) {
   require_component(c);
   for (int i=0;i<num_chunks;i++)
     chunks[i]->initialize_field(c, func);
-  step_boundaries(H_stuff);
-  step_boundaries(B_stuff);
-  step_boundaries(E_stuff);
-  step_boundaries(D_stuff);
+  step_boundaries(type(c));
+  if (is_D(c)) {
+    update_eh(E_stuff);
+    step_boundaries(E_stuff);
+  }
+  if (is_B(c)) {
+    update_eh(H_stuff);
+    step_boundaries(H_stuff);
+  }
 }
 
 void fields_chunk::initialize_field(component c, complex<double> func(const vec &)) {
   if (f[c][0]) {
-    LOOP_OVER_VOL(v, c, i) {
-      IVEC_LOOP_LOC(v, here);
+    LOOP_OVER_VOL(gv, c, i) {
+      IVEC_LOOP_LOC(gv, here);
       complex<double> val = func(here);
       f[c][0][i] += real(val);
       if (!is_real) f[c][1][i] += imag(val);
@@ -138,37 +149,4 @@ void fields_chunk::initialize_field(component c, complex<double> func(const vec
   }
 }
 
-static complex<double> (*A_static)(component, const vec &);
-static component c_static;
-static complex<double> stupid_A(const vec &p) {
-  return A_static(c_static, p);
-}
-static fields *f_static;
-static complex<double> prefactor_static;
-static complex<double> stupider_A(const vec &p) {
-  return prefactor_static*f_static->get_field(c_static, p);
-}
-
-void fields::initialize_A(complex<double> A(component, const vec &), double freq) {
-  fields tmp(*this);
-  tmp.zero_fields();
-  A_static = A;
-  FOR_ELECTRIC_COMPONENTS(ec) {
-    c_static = ec;
-    tmp.initialize_field(ec, stupid_A);
-  }
-  tmp.step();
-  f_static = &tmp;
-  const complex<double> I = complex<double>(0.0,1.0);
-  const complex<double> h_prefac = sqrt(a);
-  const complex<double> d_prefac = h_prefac / (I*freq*(2*pi));
-  FOR_COMPONENTS(c) {
-    c_static = c;
-    prefactor_static = (is_magnetic(c))? h_prefac : d_prefac;
-    initialize_field(c, stupider_A);
-  }
-  update_e_from_d();
-  step_boundaries(E_stuff);
-}
-
 } // namespace meep
