@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology.
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "meep_internals.hpp"
 
 /* This file contains a generic function for looping over all of the
-   points in all of the chunks that intersect some given volume.  This
+   points in all of the chunks that intersect some given grid_volume.  This
    is used for everything from HDF5 output to applying source volumes to
    integrating energy and flux.  It's fairly tricky because of the
    parallelization, arbitrary chunk divisions, symmetries, and periodic
@@ -35,7 +35,7 @@
    computation, and so we also perform the additional task of calculating
    the integration weights for each point -- mainly, this involves weighting
    the boundary points appropriately so that the sum approximates (via
-   linear interpolation) a continuous integral over the supplied volume. */
+   linear interpolation) a continuous integral over the supplied grid_volume. */
 
 /****************************************************************************
 
@@ -179,38 +179,80 @@ integral = f_i [-w0^3/3 + (i+1) w0^2/2
 
 ****************************************************************************/
 
+using namespace std;
+
 namespace meep {
 
+/***************************************************************/
+/* get_field_components is a utility routine, designed to be   */
+/* called by chunkloop functions, for fetching values of field */
+/* components at grid points, accounting for the complications */
+/* of symmetry and yee-grid averaging.                         */
+/***************************************************************/
+chunkloop_field_components::chunkloop_field_components(fields_chunk *fc, component cgrid,
+                                                       std::complex<double> shift_phase,
+                                                       const symmetry &S, int sn,
+                                                       int num_fields, const component *components) :
+  fc(fc), parent_components(num_fields), phases(num_fields), offsets(2*num_fields), values(num_fields)
+{
+  // for each requested component, get symmetry-parent component, yee-grid offsets, and phase shift
+  for (int nc=0; nc < num_fields; nc++) {
+    parent_components[nc] = S.transform(components[nc], -sn);
+    phases[nc] = shift_phase * S.phase_shift(parent_components[nc], sn);
+    ptrdiff_t ofs1=0, ofs2=0;
+    if (cgrid == Centered)
+      fc->gv.yee2cent_offsets(parent_components[nc], ofs1, ofs2);
+    offsets[2*nc]   = ofs1;
+    offsets[2*nc+1] = ofs2;
+  }
+}
+
+void chunkloop_field_components::update_values(ptrdiff_t idx)
+{
+  for (size_t nc=0; nc < values.size(); nc++) {
+    // do appropriate averaging to get value of field component at grid point
+    component cparent = parent_components[nc];
+    ptrdiff_t ofs1 = offsets[2*nc], ofs2 = offsets[2*nc+1];
+    double favg[2]={0.0,0.0}; // real, imag parts
+    for (int reim=0; reim<2; reim++) {
+      const double *fgrid = fc->f[cparent][reim];
+      if (!fgrid) continue;
+      favg[reim] = 0.25 * (fgrid[idx] + fgrid[idx+ofs1] + fgrid[idx+ofs2] + fgrid[idx+ofs1+ofs2]);
+    }
+    values[nc] = phases[nc] * std::complex<double>(favg[0], favg[1]);
+  }
+}
+
 /* The following two functions convert a vec to the nearest ivec
    in the dielectric (odd-coordinate) grid, either rounding down (floor)
    or up (ceil).  In the special case where a component of the vec is
    *exactly* on a component of the ivec, we add the corresponding
    component of equal_shift (which should be either -2, 0, or +2).
    (equal_shift is there to prevent us from counting edge points twice.) */
-   
-static ivec vec2diel_floor(const vec &v, double a, const ivec &equal_shift) {
-  ivec iv(v.dim);
-  LOOP_OVER_DIRECTIONS(v.dim, d) {
-    iv.set_direction(d, 1+2*int(floor(v.in_direction(d)*a-.5)));
-    if (iv.in_direction(d) == v.in_direction(d))
-      iv.set_direction(d, iv.in_direction(d) + equal_shift.in_direction(d));
+
+static ivec vec2diel_floor(const vec &pt, double a, const ivec &equal_shift) {
+  ivec ipt(pt.dim);
+  LOOP_OVER_DIRECTIONS(pt.dim, d) {
+    ipt.set_direction(d, 1+2*int(floor(pt.in_direction(d)*a-.5)));
+    if (ipt.in_direction(d) == pt.in_direction(d))
+      ipt.set_direction(d, ipt.in_direction(d) + equal_shift.in_direction(d));
   }
-  return iv;
+  return ipt;
 }
-static ivec vec2diel_ceil(const vec &v, double a, const ivec &equal_shift) {
-  ivec iv(v.dim);
-  LOOP_OVER_DIRECTIONS(v.dim, d) {
-    iv.set_direction(d, 1+2*int(ceil(v.in_direction(d)*a-.5)));
-    if (iv.in_direction(d) == v.in_direction(d))
-      iv.set_direction(d, iv.in_direction(d) + equal_shift.in_direction(d));
+static ivec vec2diel_ceil(const vec &pt, double a, const ivec &equal_shift) {
+  ivec ipt(pt.dim);
+  LOOP_OVER_DIRECTIONS(pt.dim, d) {
+    ipt.set_direction(d, 1+2*int(ceil(pt.in_direction(d)*a-.5)));
+    if (ipt.in_direction(d) == pt.in_direction(d))
+      ipt.set_direction(d, ipt.in_direction(d) + equal_shift.in_direction(d));
   }
-  return iv;
+  return ipt;
 }
 
 static inline int iabs(int i) { return (i < 0 ? -i : i); }
 
 /* Generic function for computing loops within the chunks, often
-   integral-like things, over a volume WHERE.  The job of this
+   integral-like things, over a grid_volume WHERE.  The job of this
    function is to call CHUNKLOOP() for each chunk that intersects
    WHERE, passing it the chunk, the range of integer coordinates to
    loop over, the integration weights for the boundary points, and the
@@ -219,17 +261,17 @@ static inline int iabs(int i) { return (i < 0 ? -i : i); }
    we apply the symmetry first to the chunk, *then* the shift.)
 
    We also pass CHUNKLOOP() dV0 and dV1, such that the integration
-   "volume" dV is dV0 + dV1 * iloopR, where iloopR is the loop
+   "grid_volume" dV is dV0 + dV1 * iloopR, where iloopR is the loop
    variable (starting from 0 at the starting integer coord and
    incrementing by 1) corresponding to the direction R.  Note that, in
    the LOOP_OVER_IVECS macro, iloopR corresponds to the loop variable
    loop_i2 in Dcyl (cylindrical coordinates).  In other coordinates,
-   dV1 is 0.  Note also that by "volume" dV we mean the integration
+   dV1 is 0.  Note also that by "grid_volume" dV we mean the integration
    unit corresponding to the dimensionality of WHERE (e.g. an area if
    WHERE is 2d, etc.)
 
    In particular, the loop's point coordinates are calculated on the
-   Yee grid for component cgrid.  cgrid == Dielectric is a good choice
+   Yee grid for component cgrid.  cgrid == Centered is a good choice
    if you want to work with a combination of multiple field
    components, because all of the field components can be interpolated
    onto this grid without communication between chunks.
@@ -245,50 +287,50 @@ static inline int iabs(int i) { return (i < 0 ? -i : i); }
    operations.  If use_symmetry is false, then we do *not* loop over
    all possible symmetry transformations of the chunks to see if they
    intersect WHERE; we only use chunks that, untransformed, already
-   intersect the volume.  If SNAP_EMPTY_DIMS is true, then for empty
+   intersect the grid_volume.  If SNAP_EMPTY_DIMS is true, then for empty
    (min = max) dimensions of WHERE, instead of interpolating, we
    "snap" them to the nearest grid point.  */
 
 void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
-			    const geometric_volume &where, 
+			    const volume &where,
 			    component cgrid,
 			    bool use_symmetry, bool snap_empty_dims)
 {
-  if (coordinate_mismatch(v.dim, cgrid))
+  if (coordinate_mismatch(gv.dim, cgrid))
     abort("Invalid fields::loop_in_chunks grid type %s for dimensions %s\n",
-	  component_name(cgrid), dimension_name(v.dim));
-  if (where.dim != v.dim)
+	  component_name(cgrid), dimension_name(gv.dim));
+  if (where.dim != gv.dim)
     abort("Invalid dimensions %d for WHERE in fields::loop_in_chunks", where.dim);
 
-  if (cgrid == Permeability) cgrid = Dielectric;
-  
+  if (cgrid == Permeability) cgrid = Centered;
+
   /*
     We handle looping on an arbitrary component grid by shifting
-    to the dielectric grid and then shifting back.  The looping
+    to the centered grid and then shifting back.  The looping
     coordinates are internally calculated on the odd-indexed
-    "dielectric grid", which has the virtue that it is disjoint for
+    "centered grid", which has the virtue that it is disjoint for
     each chunk and each chunk has enough information to interpolate all
     of its field components onto this grid without communication.
     Another virtue of this grid is that it is invariant under all of
     our symmetry transformations, so we can uniquely decide which
-    transformed chunk gets to loop_in_chunks which grid point. 
+    transformed chunk gets to loop_in_chunks which grid point.
   */
-  vec yee_c(v.yee_shift(Dielectric) - v.yee_shift(cgrid));
-  ivec iyee_c(v.iyee_shift(Dielectric) - v.iyee_shift(cgrid));
-  geometric_volume wherec(where + yee_c);
+  vec yee_c(gv.yee_shift(Centered) - gv.yee_shift(cgrid));
+  ivec iyee_c(gv.iyee_shift(Centered) - gv.iyee_shift(cgrid));
+  volume wherec(where + yee_c);
 
   /* Find the corners (is and ie) of the smallest bounding box for
      wherec, on the grid of odd-coordinate ivecs (i.e. the
      "epsilon grid"). */
-  ivec is(vec2diel_floor(wherec.get_min_corner(), v.a, zero_ivec(v.dim)));
-  ivec ie(vec2diel_ceil(wherec.get_max_corner(), v.a, zero_ivec(v.dim)));
-  
+  ivec is(vec2diel_floor(wherec.get_min_corner(), gv.a, zero_ivec(gv.dim)));
+  ivec ie(vec2diel_ceil(wherec.get_max_corner(), gv.a, zero_ivec(gv.dim)));
+
   /* Integration weights at boundaries (c.f. long comment at top). */
-  vec s0(v.dim), e0(v.dim), s1(v.dim), e1(v.dim);
-  LOOP_OVER_DIRECTIONS(v.dim, d) {
+  vec s0(gv.dim), e0(gv.dim), s1(gv.dim), e1(gv.dim);
+  LOOP_OVER_DIRECTIONS(gv.dim, d) {
     double w0, w1;
-    w0 = 1. - wherec.in_direction_min(d)*v.a + 0.5*is.in_direction(d);
-    w1 = 1. + wherec.in_direction_max(d)*v.a - 0.5*ie.in_direction(d);
+    w0 = 1. - wherec.in_direction_min(d)*gv.a + 0.5*is.in_direction(d);
+    w1 = 1. + wherec.in_direction_max(d)*gv.a - 0.5*ie.in_direction(d);
     if (ie.in_direction(d) >= is.in_direction(d) + 3*2) {
       s0.set_direction(d, w0*w0 / 2);
       s1.set_direction(d, 1 - (1-w0)*(1-w0) / 2);
@@ -305,8 +347,8 @@ void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
       if (snap_empty_dims) {
 	if (w0 > w1) ie.set_direction(d, is.in_direction(d));
 	else is.set_direction(d, ie.in_direction(d));
-	wherec.set_direction_min(d, is.in_direction(d) * (0.5*v.inva));
-	wherec.set_direction_max(d, is.in_direction(d) * (0.5*v.inva));
+	wherec.set_direction_min(d, is.in_direction(d) * (0.5*gv.inva));
+	wherec.set_direction_max(d, is.in_direction(d) * (0.5*gv.inva));
 	w0 = w1 = 1.0;
       }
       s0.set_direction(d, w0);
@@ -329,26 +371,26 @@ void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
     component cS = S.transform(cgrid, -sn);
     ivec iyee_cS(S.transform_unshifted(iyee_c, -sn));
 
-    geometric_volume vS = S.transform(v.surroundings(), sn);
-    vec L(v.dim);
-    ivec iL(v.dim);
+    volume gvS = S.transform(gv.surroundings(), sn);
+    vec L(gv.dim);
+    ivec iL(gv.dim);
 
     // n.b. we can't just S.transform(lattice_vector,sn), 'cause of signs
-    LOOP_OVER_DIRECTIONS(v.dim, d) {
+    LOOP_OVER_DIRECTIONS(gv.dim, d) {
       direction dS = S.transform(d, -sn).d;
       L.set_direction(d, fabs(lattice_vector(dS).in_direction(dS)));
       iL.set_direction(d, iabs(ilattice_vector(dS).in_direction(dS)));
     }
 
-    // figure out range of lattice shifts for which vS intersects wherec:
-    ivec min_ishift(v.dim), max_ishift(v.dim);
-    LOOP_OVER_DIRECTIONS(v.dim, d) {
+    // figure out range of lattice shifts for which gvS intersects wherec:
+    ivec min_ishift(gv.dim), max_ishift(gv.dim);
+    LOOP_OVER_DIRECTIONS(gv.dim, d) {
       if (boundaries[High][S.transform(d, -sn).d] == Periodic) {
-	min_ishift.set_direction(d, 
-	 int(floor((wherec.in_direction_min(d) - vS.in_direction_max(d))
+	min_ishift.set_direction(d,
+	 int(floor((wherec.in_direction_min(d) - gvS.in_direction_max(d))
 		   / L.in_direction(d))));
 	max_ishift.set_direction(d,
-	 int(ceil((wherec.in_direction_max(d) - vS.in_direction_min(d))
+	 int(ceil((wherec.in_direction_max(d) - gvS.in_direction_min(d))
 		  / L.in_direction(d))));
       }
       else {
@@ -356,14 +398,14 @@ void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
 	max_ishift.set_direction(d, 0);
       }
     }
-    
+
     // loop over lattice shifts
     ivec ishift(min_ishift);
     do {
       complex<double> ph = 1.0;
-      vec shift(v.dim, 0.0);
-      ivec shifti(v.dim, 0);
-      LOOP_OVER_DIRECTIONS(v.dim, d) {
+      vec shift(gv.dim, 0.0);
+      ivec shifti(gv.dim, 0);
+      LOOP_OVER_DIRECTIONS(gv.dim, d) {
 	shift.set_direction(d, L.in_direction(d) * ishift.in_direction(d));
 	shifti.set_direction(d, iL.in_direction(d) * ishift.in_direction(d));
 	ph *= pow(eikna[d], ishift.in_direction(d));
@@ -372,33 +414,33 @@ void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
       for (int i = 0; i < num_chunks; ++i) {
 	if (!chunks[i]->is_mine()) continue;
 	// Chunk looping boundaries:
-	geometric_volume gvS(v.dim);
+	volume vS(gv.dim);
 
 	if (use_symmetry)
-	  gvS = S.transform(chunks[i]->gv, sn);
+	  vS = S.transform(chunks[i]->v, sn);
 	else {
 	  /* If we're not using symmetry, it's because (as in src_vol)
 	     we don't care about correctly counting the points in the
-	     volume.  Rather, we just want to make sure to get *all*
+	     grid_volume.  Rather, we just want to make sure to get *all*
 	     of the chunk points that intersect where.  Hence, add a little
 	     padding to make sure we don't miss any points due to rounding. */
-	  vec pad(one_ivec(v.dim) * v.inva * 1e-3);
-	  gvS = geometric_volume(chunks[i]->v.loc(Dielectric,0) - pad,
-				 chunks[i]->v.loc(Dielectric,
-						  chunks[i]->v.ntot()-1) +pad);
+	  vec pad(one_ivec(gv.dim) * gv.inva * 1e-3);
+	  vS = volume(chunks[i]->gv.loc(Centered,0) - pad,
+				 chunks[i]->gv.loc(Centered,
+						  chunks[i]->gv.ntot()-1) +pad);
 	}
 
-	ivec iscS(max(is-shifti, vec2diel_ceil(gvS.get_min_corner(),
-					       v.a, one_ivec(v.dim) * 2)));
-	ivec iecS(min(ie-shifti, vec2diel_floor(gvS.get_max_corner(),
-						v.a, zero_ivec(v.dim))));
+	ivec iscS(max(is-shifti, vec2diel_ceil(vS.get_min_corner(),
+					       gv.a, one_ivec(gv.dim) * 2)));
+	ivec iecS(min(ie-shifti, vec2diel_floor(vS.get_max_corner(),
+						gv.a, zero_ivec(gv.dim))));
 	if (iscS <= iecS) {
 	  // Determine weights at chunk looping boundaries:
 	  ivec isc(S.transform(iscS, -sn)), iec(S.transform(iecS, -sn));
-	  vec s0c(v.dim,1.0), s1c(v.dim,1.0), e0c(v.dim,1.0), e1c(v.dim,1.0);
+	  vec s0c(gv.dim,1.0), s1c(gv.dim,1.0), e0c(gv.dim,1.0), e1c(gv.dim,1.0);
 	  iscS += shifti;
 	  iecS += shifti;
-	  LOOP_OVER_DIRECTIONS(v.dim, d) {
+	  LOOP_OVER_DIRECTIONS(gv.dim, d) {
 	    direction dS = S.transform(d, sn).d;
 	    if (iscS.in_direction(dS) == is.in_direction(dS)) {
 	      s0c.set_direction(d, s0.in_direction(dS));
@@ -443,19 +485,19 @@ void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
 	      e1c.set_direction(d, swap);
 	    }
 	  }
-	  
+
 
 	  // Determine integration "volumes" dV0 and dV1;
 	  double dV0 = 1.0, dV1 = 0.0;
-	  LOOP_OVER_DIRECTIONS(v.dim, d)
+	  LOOP_OVER_DIRECTIONS(gv.dim, d)
 	    if (wherec.in_direction(d) > 0.0)
-	      dV0 *= v.inva;
-	  if (v.dim == Dcyl) {
-	    dV1 = dV0 * 2*pi * v.inva;
-	    dV0 *= 2*pi * fabs((S.transform(chunks[i]->v[isc], sn) + shift
+	      dV0 *= gv.inva;
+	  if (gv.dim == Dcyl) {
+	    dV1 = dV0 * 2*pi * gv.inva;
+	    dV0 *= 2*pi * fabs((S.transform(chunks[i]->gv[isc], sn) + shift
 				- yee_c).in_direction(R));
 	  }
-	 
+
 	  chunkloop(chunks[i], i, cS,
 		    isc - iyee_cS, iec - iyee_cS,
 		    s0c, s1c, e0c, e1c,
@@ -465,9 +507,9 @@ void fields::loop_in_chunks(field_chunkloop chunkloop, void *chunkloop_data,
 		    chunkloop_data);
 	}
       }
-      
-      
-      LOOP_OVER_DIRECTIONS(v.dim, d) {
+
+
+      LOOP_OVER_DIRECTIONS(gv.dim, d) {
 	if (ishift.in_direction(d) + 1 <= max_ishift.in_direction(d)) {
 	  ishift.set_direction(d, ishift.in_direction(d) + 1);
 	  break;
