@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #define MEEP_MY_MPI_H
 
 #include <complex>
-using namespace std;
+#include <stddef.h>
 
 namespace meep {
 
@@ -40,38 +40,54 @@ class initialize {
 #  define NORETURN_ATTR __attribute__((noreturn))
 #  define PRINTF_ATTR(f,a) __attribute__((format(printf, f, a)))
 #else
-#  define NORETURN_ATTR 
-#  define PRINTF_ATTR(f,a) 
+#  define NORETURN_ATTR
+#  define PRINTF_ATTR(f,a)
 #endif
 
 void abort(const char *fmt, ...) NORETURN_ATTR PRINTF_ATTR(1,2);
 void all_wait();
 int count_processors();
 int my_rank();
+bool am_really_master();
 inline int am_master() { return my_rank() == 0; }
+bool with_mpi();
 
 void send(int from, int to, double *data, int size=1);
 void broadcast(int from, double *data, int size);
 void broadcast(int from, char *data, int size);
 void broadcast(int from, int *data, int size);
-void broadcast(int from, complex<double> *data, int size);
-complex<double> broadcast(int from, complex<double> data);
+void broadcast(int from, size_t *data, int size);
+void broadcast(int from, std::complex<double> *data, int size);
+std::complex<double> broadcast(int from, std::complex<double> data);
 double broadcast(int from, double data);
 int broadcast(int from, int data);
 bool broadcast(int from, bool);
 double max_to_master(double); // Only returns the correct value to proc 0.
 double max_to_all(double);
 int max_to_all(int);
+float sum_to_master(float); // Only returns the correct value to proc 0.
 double sum_to_master(double); // Only returns the correct value to proc 0.
 double sum_to_all(double);
 void sum_to_all(const double *in, double *out, int size);
+void sum_to_master(const float *in, float *out, int size);
+void sum_to_master(const double *in, double *out, int size);
+void sum_to_all(const float *in, double *out, int size);
+void sum_to_all(const std::complex<float> *in, std::complex<double> *out, int size);
+void sum_to_all(const std::complex<double> *in, std::complex<double> *out, int size);
+void sum_to_master(const std::complex<float> *in, std::complex<float> *out, int size);
+void sum_to_master(const std::complex<double> *in, std::complex<double> *out, int size);
 long double sum_to_all(long double);
-complex<double> sum_to_all(complex<double> in);
-complex<long double> sum_to_all(complex<long double> in);
+std::complex<double> sum_to_all(std::complex<double> in);
+std::complex<long double> sum_to_all(std::complex<long double> in);
 int sum_to_all(int);
 int partial_sum_to_all(int in);
+size_t sum_to_all(size_t);
+size_t partial_sum_to_all(size_t in);
+void sum_to_all(const size_t *in, size_t *out, int size);
+void sum_to_master(const size_t *in, size_t *out, int size);
 bool or_to_all(bool in);
 void or_to_all(const int *in, int *out, int size);
+void bw_or_to_all(const size_t *in, size_t *out, int size);
 bool and_to_all(bool in);
 void and_to_all(const int *in, int *out, int size);
 
@@ -85,6 +101,13 @@ void master_fclose(FILE *f);
 void begin_critical_section(int tag);
 void end_critical_section(int tag);
 
+int divide_parallel_processes(int numgroups);
+void begin_global_communications(void);
+void end_global_communications(void);
+void end_divide_parallel(void);
+
+int my_global_rank(void);
+
 } /* namespace meep */
 
 #endif /* MEEP_MY_MPI_H */
