@@ -1,5 +1,5 @@
 // -*- C++ -*-
-/* Copyright (C) 2005-2014 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,7 @@
 #define MEEP_VEC_H
 
 #include <complex>
-
-using namespace std;
+#include <stddef.h>
 
 namespace meep {
 
@@ -38,20 +37,20 @@ enum field_type { E_stuff=0, H_stuff=1, D_stuff=2, B_stuff=3, PE_stuff=4, PH_stu
 enum boundary_side { High=0, Low };
 enum direction { X=0,Y,Z,R,P, NO_DIRECTION };
 struct signed_direction {
-  signed_direction(direction dd=X,bool f=false, complex<double> ph=1.0) {
+  signed_direction(direction dd=X,bool f=false, std::complex<double> ph=1.0) {
     d = dd; flipped = f; phase = ph;
   };
   signed_direction(const signed_direction &sd) {
     d = sd.d; flipped = sd.flipped; phase = sd.phase;
   }
-  signed_direction operator*(complex<double> ph);
+  signed_direction operator*(std::complex<double> ph);
   bool operator==(const signed_direction &sd) const { return (d == sd.d &&
 						       flipped == sd.flipped
 						       && phase == sd.phase); }
   bool operator!=(const signed_direction &sd) const { return !(*this == sd); }
   direction d;
   bool flipped;
-  complex<double> phase;
+  std::complex<double> phase;
 };
 
 inline int number_of_directions(ndim dim) {
@@ -121,7 +120,7 @@ component first_field_component(field_type ft);
 
 // loop over indices idx from is to ie (inclusive) in gv
 #define LOOP_OVER_IVECS(gv, is, ie, idx) \
-  for (int loop_is1 = (is).yucky_val(0), \
+  for (ptrdiff_t loop_is1 = (is).yucky_val(0), \
            loop_is2 = (is).yucky_val(1), \
            loop_is3 = (is).yucky_val(2), \
            loop_n1 = ((ie).yucky_val(0) - loop_is1) / 2 + 1, \
@@ -138,7 +137,7 @@ component first_field_component(field_type ft);
                 + (is - (gv).little_corner()).yucky_val(2) / 2 * loop_s3,\
            loop_i1 = 0; loop_i1 < loop_n1; loop_i1++) \
     for (int loop_i2 = 0; loop_i2 < loop_n2; loop_i2++) \
-      for (int idx = idx0 + loop_i1*loop_s1 + loop_i2*loop_s2, \
+      for (ptrdiff_t idx = idx0 + loop_i1*loop_s1 + loop_i2*loop_s2, \
            loop_i3 = 0; loop_i3 < loop_n3; loop_i3++, idx+=loop_s3)
 
 #define LOOP_OVER_VOL(gv, c, idx) \
@@ -172,9 +171,17 @@ component first_field_component(field_type ft);
 // should only use these macros where that is true!  (Basically,
 // all of this is here to support performance hacks of step_generic.)
 
+#if !defined(__INTEL_COMPILER) && !defined(__clang__) && (defined(__GNUC__) || defined(__GNUG__))
+# define IVDEP _Pragma("GCC ivdep")
+#elif defined(__INTEL_COMPILER)
+# define IVDEP _Pragma("ivdep")
+#else
+# define IVDEP
+#endif
+
 // loop over indices idx from is to ie (inclusive) in gv
 #define S1LOOP_OVER_IVECS(gv, is, ie, idx) \
-  for (int loop_is1 = (is).yucky_val(0), \
+  for (ptrdiff_t loop_is1 = (is).yucky_val(0), \
            loop_is2 = (is).yucky_val(1), \
            loop_is3 = (is).yucky_val(2), \
            loop_n1 = ((ie).yucky_val(0) - loop_is1) / 2 + 1, \
@@ -189,8 +196,8 @@ component first_field_component(field_type ft);
                 + (is - (gv).little_corner()).yucky_val(1) / 2 * loop_s2 \
                 + (is - (gv).little_corner()).yucky_val(2) / 2 * loop_s3,\
            loop_i1 = 0; loop_i1 < loop_n1; loop_i1++) \
-    for (int loop_i2 = 0; loop_i2 < loop_n2; loop_i2++) _Pragma("ivdep") \
-      for (int idx = idx0 + loop_i1*loop_s1 + loop_i2*loop_s2, \
+    for (int loop_i2 = 0; loop_i2 < loop_n2; loop_i2++) IVDEP \
+      for (ptrdiff_t idx = idx0 + loop_i1*loop_s1 + loop_i2*loop_s2, \
            loop_i3 = 0; loop_i3 < loop_n3; loop_i3++, idx++)
 
 #define S1LOOP_OVER_VOL(gv, c, idx) \
@@ -249,7 +256,7 @@ inline bool has_field_direction(ndim dim, direction d) {
   return false;
 }
 
-// true if d is polar while dim is cartesian, or vice versa 
+// true if d is polar while dim is cartesian, or vice versa
 inline bool coordinate_mismatch(ndim dim, direction d) {
   return (d != NO_DIRECTION &&
 	  ((dim >= D1 && dim <= D3 && d != X && d != Y && d != Z) ||
@@ -282,7 +289,7 @@ const char *direction_name(direction);
 const char *dimension_name(ndim);
 
 inline int component_index(component c) {
-  switch (c) {    
+  switch (c) {
   case Ex: case Hx: case Dx: case Bx: return 0;
   case Ey: case Hy: case Dy: case By: return 1;
   case Ez: case Hz: case Dz: case Bz: return 2;
@@ -297,7 +304,7 @@ inline int component_index(component c) {
 direction component_direction(int c);
 int direction_component(int c, direction d);
 inline direction component_direction(component c) {
-  switch (c) {    
+  switch (c) {
   case Ex: case Hx: case Dx: case Bx: return X;
   case Ey: case Hy: case Dy: case By: return Y;
   case Ez: case Hz: case Dz: case Bz: return Z;
@@ -519,11 +526,11 @@ ivec one_ivec(ndim);
 
 class ivec {
  public:
-  ivec() { dim = D2; t[X] = t[Y] = 0; };
-  ivec(ndim di) { dim = di; };
+  ivec() { dim = D2; t[X]=t[Y]=t[Z]=0; };
+  ivec(ndim di) { dim = di; t[X]=t[Y]=t[Z]=0; };
   ivec(ndim di, int val) { dim = di; t[0]=t[1]=t[2]=t[3]=t[4]=val; };
-  ivec(int zz) { dim = D1; t[Z] = zz; };
-  ivec(int xx, int yy) { dim = D2; t[X] = xx; t[Y] = yy; };
+  ivec(int zz) { dim = D1; t[X]=t[Y]=0; t[Z] = zz; };
+  ivec(int xx, int yy) { dim = D2; t[X] = xx; t[Y] = yy; t[Z]=0; };
   ivec(int xx, int yy, int zz) {
     dim = D3; t[X] = xx; t[Y] = yy; t[Z] = zz; };
   friend ivec iveccyl(int xx, int yy);
@@ -611,10 +618,10 @@ class ivec {
   int in_direction(direction d) const { return t[d]; };
   void set_direction(direction d, int val) { t[d] = val; };
 
-  ivec round_up_to_even(void) const { 
+  ivec round_up_to_even(void) const {
     ivec result(dim);
-    LOOP_OVER_DIRECTIONS(dim, d) 
-      result.t[d] = t[d] + (t[d] >= 0 ? t[d] : -t[d]) % 2; 
+    LOOP_OVER_DIRECTIONS(dim, d)
+      result.t[d] = t[d] + (t[d] >= 0 ? t[d] : -t[d]) % 2;
     return result;
   }
 
@@ -656,12 +663,13 @@ class volume {
   volume(ndim di) { dim = di; min_corner.dim = di; max_corner.dim = di; };
   volume(const vec &vec1, const vec &vec2);
   volume(const vec &pt);
+  volume(const volume &vol);
   void set_direction_min(direction d, double val) { min_corner.set_direction(d, val); };
   void set_direction_max(direction d, double val) { max_corner.set_direction(d, val); };
   double in_direction_min(direction d) const { return min_corner.in_direction(d); };
   double in_direction_max(direction d) const { return max_corner.in_direction(d); };
   double in_direction(direction d) const { return in_direction_max(d) - in_direction_min(d); }
-  double computational_volume() const; 
+  double computational_volume() const;
   double integral_volume() const;
   double full_volume() const;
   vec center() const { return (min_corner + max_corner) * 0.5; }
@@ -724,7 +732,7 @@ class grid_volume {
   double a, inva /* = 1/a */;
 
   void print() const;
-  int stride(direction d) const { return the_stride[d]; };
+  ptrdiff_t stride(direction d) const { return the_stride[d]; };
   int num_direction(direction d) const {
     return num[((int) d) % 3];
   };
@@ -749,16 +757,16 @@ class grid_volume {
   vec dy() const;
   vec dz() const;
 
-  int ntot() const { return the_ntot; }
-  int nowned_min() const { int n = 1; LOOP_OVER_DIRECTIONS(dim,d) n *= num_direction(d); return n; }
-  int nowned(component c) const;
+  size_t ntot() const { return the_ntot; }
+  size_t nowned_min() const { size_t n = 1; LOOP_OVER_DIRECTIONS(dim,d) n *= (size_t)(num_direction(d)); return n; }
+  size_t nowned(component c) const;
   vec operator[](const ivec &p) const { return p*(0.5*inva); };
-  int index(component, const ivec &) const;
+  ptrdiff_t index(component, const ivec &) const;
   ivec round_vec(const vec &) const;
-  void interpolate(component, const vec &, int indices[8], double weights[8]) const;
+  void interpolate(component, const vec &, ptrdiff_t indices[8], double weights[8]) const;
   void interpolate(component, const vec &, ivec locs[8], double weights[8]) const;
 
-  volume dV(component c, int index) const;
+  volume dV(component c, ptrdiff_t index) const;
   volume dV(const ivec &, double diameter = 1.0) const;
   bool intersect_with(const grid_volume &vol_in, grid_volume *intersection = NULL, grid_volume *others = NULL, int *num_others = NULL) const;
   double rmin() const;
@@ -771,21 +779,21 @@ class grid_volume {
   double zmax() const;
   vec center() const;
   ivec icenter() const;
-  vec loc(component, int index) const;
-  vec loc_at_resolution(int index, double res) const;
-  int ntot_at_resolution(double res) const;
-  ivec iloc(component, int index) const;
+  vec loc(component, ptrdiff_t index) const;
+  vec loc_at_resolution(ptrdiff_t index, double res) const;
+  size_t ntot_at_resolution(double res) const;
+  ivec iloc(component, ptrdiff_t index) const;
 
-  int yee_index(component c) const {
-    int idx = 0;
+  ptrdiff_t yee_index(component c) const {
+    ptrdiff_t idx = 0;
     LOOP_OVER_DIRECTIONS(dim,d)
       idx += (1-iyee_shift(c).in_direction(d))*stride(d);
     return idx;
   }
   vec yee_shift(component) const;
   component eps_component() const;
-  void yee2cent_offsets(component c, int &offset1, int &offset2) const;
-  void cent2yee_offsets(component c, int &offset1, int &offset2) const;
+  void yee2cent_offsets(component c, ptrdiff_t &offset1, ptrdiff_t &offset2) const;
+  void cent2yee_offsets(component c, ptrdiff_t &offset1, ptrdiff_t &offset2) const;
 
   double boundary_location(boundary_side, direction) const;
   ivec big_corner() const;
@@ -815,7 +823,7 @@ class grid_volume {
   friend grid_volume vol2d(double xsize, double ysize, double a);
   friend grid_volume vol3d(double xsize, double ysize, double zsize, double a);
 
-  grid_volume split(int num, int which) const;
+  grid_volume split(size_t num, int which) const;
   grid_volume split_by_effort(int num, int which, int Ngv = 0, const grid_volume *v = NULL, double *effort = NULL) const;
   grid_volume split_at_fraction(bool want_high, int numer) const;
   grid_volume halve(direction d) const;
@@ -830,7 +838,7 @@ class grid_volume {
   ivec iyee_shift(component c) const {
     ivec out = zero_ivec(dim);
     LOOP_OVER_DIRECTIONS(dim,d)
-      if (c == Dielectric || c == Permeability || 
+      if (c == Dielectric || c == Permeability ||
           ((is_electric(c) || is_D(c)) && d == component_direction(c)) ||
           ((is_magnetic(c) || is_B(c)) && d != component_direction(c)))
         out.set_direction(d,1);
@@ -860,8 +868,8 @@ class grid_volume {
   void set_strides();
   void num_changed() { update_ntot(); set_strides(); }
   int num[3];
-  int the_stride[5];
-  int the_ntot;
+  ptrdiff_t the_stride[5];
+  size_t the_ntot;
 };
 
 class volume_list;
@@ -889,18 +897,18 @@ class symmetry {
   ivec transform_unshifted(const ivec &, int n) const;
   volume transform(const volume &, int n) const;
   component transform(component, int n) const;
-  complex<double> phase_shift(component, int n) const;
+  std::complex<double> phase_shift(component, int n) const;
   derived_component transform(derived_component, int n) const;
-  complex<double> phase_shift(derived_component, int n) const;
+  std::complex<double> phase_shift(derived_component, int n) const;
   int transform(int, int n) const;
-  complex<double> phase_shift(int, int n) const;
+  std::complex<double> phase_shift(int, int n) const;
   int multiplicity() const;
   bool is_primitive(const ivec &) const;
 
   volume_list *reduce(const volume_list *gl) const;
 
   symmetry operator+(const symmetry &) const;
-  symmetry operator*(complex<double>) const;
+  symmetry operator*(std::complex<double>) const;
   symmetry operator-(const symmetry &b) const { return *this + b * (-1.0); }
   symmetry operator-(void) const { return *this * (-1.0); }
   void operator=(const symmetry &);
@@ -908,7 +916,7 @@ class symmetry {
   bool operator!=(const symmetry &S) const { return !(*this == S); };
  private:
   signed_direction S[5];
-  complex<double> ph;
+  std::complex<double> ph;
   vec symmetry_point;
   ivec i_symmetry_point;
   int g; // g is the multiplicity of the symmetry.
@@ -918,12 +926,20 @@ class symmetry {
 
 class volume_list {
 public:
-  volume_list(const volume &v, int c, complex<double> weight = 1.0, volume_list *next = 0) : v(v), c(c), weight(weight), next(next) {}
+  volume_list(const volume &v, int c, std::complex<double> weight = 1.0, volume_list *next = 0) : v(v), c(c), weight(weight), next(next) {}
   ~volume_list() { delete next; }
-  
+  volume_list(const volume_list *vl) : v(vl->v), c(vl->c), weight(vl->weight), next(0) {
+      volume_list *p = vl->next, *q = this;
+      while (p) {
+          q->next = new volume_list(*p);
+          q = q->next;
+          p = p->next;
+      }
+  }
+
   volume v;
   int c; // component or derived component associated with v (e.g. for flux)
-  complex<double> weight;
+  std::complex<double> weight;
   volume_list *next;
 };
 
