@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008 Massachusetts Institute of Technology.
+/* Copyright (C) 2005-2015 Massachusetts Institute of Technology.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include <meep.hpp>
 using namespace meep;
+using namespace std;
 
 double the_value = 1.0;
 double value(const vec &) { return the_value; }
@@ -29,11 +30,11 @@ void harmonics(double freq, double chi2, double chi3, double J,
   const double dpml = 5.0;
   const double res = 20;
   const double sz = 100+2*dpml;
-  volume v = vol1d(sz, res);
-  v.center_origin();
+  grid_volume gv = vol1d(sz, res);
+  gv.center_origin();
 
   the_value = 1.0;
-  structure s(v, value, pml(dpml));
+  structure s(gv, value, pml(dpml));
   the_value = chi2;
   s.set_chi2(value);
   the_value = chi3;
@@ -46,9 +47,9 @@ void harmonics(double freq, double chi2, double chi3, double J,
   f.add_point_source(Ex, src, vec(-0.5 * sz + dpml), J);
 
   vec fpt(0.5 * sz - dpml - 0.5);
-  dft_flux d1 = f.add_dft_flux(Z, geometric_volume(fpt), freq, freq, 1);
-  dft_flux d2 = f.add_dft_flux(Z, geometric_volume(fpt), 2*freq, 2*freq, 1);
-  dft_flux d3 = f.add_dft_flux(Z, geometric_volume(fpt), 3*freq, 3*freq, 1);
+  dft_flux d1 = f.add_dft_flux(Z, volume(fpt), freq, freq, 1);
+  dft_flux d2 = f.add_dft_flux(Z, volume(fpt), 2*freq, 2*freq, 1);
+  dft_flux d3 = f.add_dft_flux(Z, volume(fpt), 3*freq, 3*freq, 1);
 
   double emax = 0;
   
@@ -100,10 +101,11 @@ int main(int argc, char **argv) {
 
   double a2, a3, a2_2, a3_2;
 
+  double thresh = sizeof(realnum) == sizeof(float) ? 1e-4 : 1e-5;
   harmonics(freq, 0.27e-4, 1e-4, 1.0, a2, a3);
-  if (different(a2, 9.79875e-07, 1e-5, "2nd harmonic mismatches known value"))
+  if (different(a2, 9.80330e-07, thresh, "2nd harmonic mismatches known val"))
     return 1;
-  if (different(a3, 9.97972e-07, 1e-5, "3rd harmonic mismatches known value"))
+  if (different(a3, 9.97747e-07, thresh, "3rd harmonic mismatches known val"))
     return 1;
 
   harmonics(freq, 0.54e-4, 2e-4, 1.0, a2_2, a3_2);
